﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.Drawing.Imaging;

namespace howto_image_hash
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Compare the images.
        private void btnCompare_Click(object sender, EventArgs e)
        {
            string hash_code1 = ProcessImage(picImage1.Image as Bitmap,
                picShrunk1, picMonochrome1, txtHashCode1);
            string hash_code2 = ProcessImage(picImage2.Image as Bitmap,
                picShrunk2, picMonochrome2, txtHashCode2);

            // Display the difference and score.
            string difference = "";
            int score = 0;
            for (int i = 0; i < hash_code1.Length; i++)
            {
                if (hash_code1[i] == hash_code2[i])
                {
                    difference += " ";
                }
                else
                {
                    difference += "X";
                    score++;
                }
            }
            txtDifference.Text = difference;
            txtScore.Text = score.ToString();
            float percent = (hash_code1.Length - score) / (float)hash_code1.Length;
            txtPercent.Text = percent.ToString("P");
        }

        private string ProcessImage(Bitmap original_bm,
            PictureBox pic_shrunk, PictureBox pic_monochrome,
            TextBox txt_hashcode)
        {
            // Shrink the original image and display the result.
            Bitmap shrunk_bm = ScaleTo(original_bm, 9, 9,
                InterpolationMode.High);
            pic_shrunk.Image = ScaleTo(shrunk_bm, 90, 90,
                InterpolationMode.NearestNeighbor);

            // Convert to grayscale and display the result.
            Bitmap grayscale_bm = ToMonochrome(shrunk_bm);
            pic_monochrome.Image = ScaleTo(grayscale_bm, 90, 90,
                InterpolationMode.NearestNeighbor);

            // Calculate the hash code.
            string hash_code = GetHashCode(grayscale_bm);
            txt_hashcode.Text = hash_code;

            return hash_code;
        }

        // Scale an image.
        private Bitmap ScaleTo(Bitmap bm, int wid, int hgt,
            InterpolationMode interpolation_mode)
        {
            Bitmap new_bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(new_bm))
            {
                RectangleF source_rect = new RectangleF(-0.5f, -0.5f, bm.Width, bm.Height);
                Rectangle dest_rect = new Rectangle(0, 0, wid, hgt);
                gr.InterpolationMode = interpolation_mode;
                gr.DrawImage(bm, dest_rect, source_rect, GraphicsUnit.Pixel);
            }
            return new_bm;
        }

        // Convert an image to monochrome.
        private Bitmap ToMonochrome(Image image)
        {
            // Make the ColorMatrix.
            ColorMatrix cm = new ColorMatrix(new float[][]
            {
                new float[] {0.299f, 0.299f, 0.299f, 0, 0},
                new float[] {0.587f, 0.587f, 0.587f, 0, 0},
                new float[] {0.114f, 0.114f, 0.114f, 0, 0},
                new float[] { 0, 0, 0, 1, 0},
                new float[] { 0, 0, 0, 0, 1}
            });
            ImageAttributes attributes = new ImageAttributes();
            attributes.SetColorMatrix(cm);

            // Draw the image onto the new bitmap while
            // applying the new ColorMatrix.
            Point[] points =
            {
                new Point(0, 0),
                new Point(image.Width, 0),
                new Point(0, image.Height),
            };
            Rectangle rect = new Rectangle(0, 0, image.Width, image.Height);

            // Make the result bitmap.
            Bitmap bm = new Bitmap(image.Width, image.Height);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.DrawImage(image, points, rect,
                    GraphicsUnit.Pixel, attributes);
            }

            // Return the result.
            return bm;
        }

        // Return the hashcode for this 9x9 image.
        private string GetHashCode(Bitmap bm)
        {
            string row_hash = "";
            for (int r = 0; r < 8; r++)
                for (int c = 0; c < 8; c++)
                    if (bm.GetPixel(c + 1, r).R >= bm.GetPixel(c, r).R)
                        row_hash += "1";
                    else
                        row_hash += "0";

            string col_hash = "";
            for (int c = 0; c < 8; c++)
                for (int r = 0; r < 8; r++)
                    if (bm.GetPixel(c, r + 1).R >= bm.GetPixel(c, r).R)
                        col_hash += "1";
                    else
                        col_hash += "0";

            return row_hash + "," + col_hash;
        }

        private void btnLoadImage1_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                picImage1.Image = LoadBitmapUnlocked(ofdImage.FileName);
            }
        }

        private void btnLoadImage2_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                picImage2.Image = LoadBitmapUnlocked(ofdImage.FileName);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }
    }
}
